// TODO: loose: all?
// TODO: external helpers?

var babel = require('babel-core')
	, extend = require('lodash/object/extend')

	, BOILERPLATE = 'var global = window.global = window;\n\n'
	, SETTINGS = {
			externalHelpers: true,
			loose: 'all',
			whitelist: [
				'es6.arrowFunctions',
				'es6.blockScoping',
				'es6.classes',
				'es6.constants',
				'es6.destructuring',
				'es6.forOf',
				'es6.modules',
				'es6.objectSuper',
				'es6.parameters',
				'es6.properties.computed',
				'es6.properties.shorthand',
				'es6.regex.sticky',
				'es6.regex.unicode',
				'es6.spread',
				'es6.tailCall',
				'es6.templateLiterals',
				'react',
				'react.displayName'/*,
				'optimisation.react.constantElements',
				'optimisation.react.inlineElements',
				'minification.deadCodeElimination'*/
			]
		};

/**
 * Retrieve registration data
 */
exports.registration = {
	name: 'babel',
	extensions: {
		js: [
			'es6',
			'es7',
			'js',
			'jsx'
		]
	}
};

/**
 * Compile 'content'
 * @param {String} content
 * @param {Object} options
 * @param {Function} fn(err, content)
 */
exports.compile = function (content, options, fn) {
	// Skip node_modules files
	if (!~options.filepath.indexOf('node_modules')) {
		try {
			var cache = options.cache
				, cachedHelpers = cache.getSource('babel-helpers') || []
				, transform = babel.transform(content, extend({}, SETTINGS))
				, helpers = (transform.metadata.usedHelpers || []).filter(function (helper) {
						return !~cachedHelpers.indexOf(helper);
					});

			if (helpers.length) {
				cachedHelpers = cachedHelpers.concat(helpers);
				// Store used helpers
				cache.setSource('babel-helpers', cachedHelpers, true);
				// Convert and store helper boilerplate
				cache.setSource('js-helpers', BOILERPLATE + babel.buildExternalHelpers(cachedHelpers), true);
			}

			fn(null, transform.code);
		} catch (err) {
			err.filepath = options.filepath;
			fn(err);
		}
	} else {
		fn(null, content);
	}
};
